--[[
Name: LibAuraInfo-1.0
Author(s): Cyprias (cyprias@gmail.com)
Documentation: http://www.wowace.com/addons/libaurainfo-1-0/
SVN: svn://svn.wowace.com/wow/libaurainfo-1-0/mainline/trunk
Description: Keep track of spell duration and debuff Type.
Dependencies: LibStub
]]

local _G = _G
local print = print
local UnitAura = UnitAura
local _
local math_floor = math.floor
local strsplit = strsplit
local tonumber = tonumber
local UnitIsUnit = UnitIsUnit
local UnitExists = UnitExists

local MAJOR, MINOR = "LibAuraInfo-1.0", 1
if not LibStub then error(MAJOR .. " requires LibStub.") return end

local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end


local DEBUG = false
--[===[@debug@
DEBUG = true
--@end-debug@]===]

local function debugPrint(...)
	if DEBUG then
		print(...)
	end
end

--Save debuffType as a number, then return as a string when requested.
local debuffTypes = {
	Magic = 1,
	Disease = 2,
	Poison = 3,
	Curse = 4,
	[1] = "Magic",
	[2] = "Disease",
	[3] = "Poison",
	[4] = "Curse",
}

--Return the duration of a spell.
function lib:GetDuration(spellID)
	if lib.auraInfo[spellID] then
		local duration, debuffType = strsplit(";", lib.auraInfo[spellID])
		return tonumber(duration or 0)
	end
	return nil
end

--Return the debuff type of a spell.
function lib:GetDebuffType(spellID)
	if lib.auraInfo[spellID] then
		local duration, debuffType = strsplit(";", lib.auraInfo[spellID])
		if debuffType then
			if debuffType == 0 then
				return "None"
			else
				return debuffTypes[debuffType] or "unknown"
			end
		end
		return "None"
	end
	return nil
end



--[===[@debug@
--Code not run by end users. This is used to find info on missing spells.
local LAI_DB
local function Round(num, zeros)
	return math_floor( num * 10 ^ (zeros or 0) + 0.5 ) / 10 ^ (zeros or 0)
end
local function CheckUnitAuras(unitID, filterType)
--~ 	name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId  = UnitAura("unit", index or "name"[, "rank"[, "filter"]]) 
	local i = 1;
	local name, rank, dispelType, duration, isStealable, spellID
	while true do 
		name, rank, _, _, dispelType, duration, _, _, isStealable, _, spellID = UnitAura(unitID, i, filterType)
		if not name then break end
		
		if not duration  then
			debugPrint("duration", spellID, name, rank, duration)
		end
		
		if isStealable then
			debugPrint("isStealable", spellID, name, rank, isStealable)
		end
		
		duration = Round(duration)
		if not lib.auraInfo[spellID] and not LAI_DB[spellID] then
			lib.auraInfo[spellID] = (duration or 0) .. ";" .. (dispelType and debuffTypes[dispelType] or 0)--add it temporarily.
			LAI_DB[spellID] = lib.auraInfo[spellID]    .."  --"..name .. (rank and rank ~= "" and " ("..rank..")" or "") .."*"
			debugPrint("Missing info on", spellID, dispelType, duration, " = ", LAI_DB[spellID])
		end
		i = i + 1
	end
end
local function OnEvent(self, event, ...)
--~ 	debugPrint(event, ...)
	if event == "UPDATE_MOUSEOVER_UNIT" then
		CheckUnitAuras("mouseover", "HELPFUL")
		CheckUnitAuras("mouseover", "HARMFUL")
	elseif event == "PLAYER_TARGET_CHANGED" then
		CheckUnitAuras("target", "HELPFUL")
		CheckUnitAuras("target", "HARMFUL")
	elseif event == "UNIT_TARGET" then
		local unitID = ...
		if not UnitIsUnit(unitID, "player") and UnitExists(unitID.."target") then
			CheckUnitAuras(unitID.."target", "HELPFUL")
			CheckUnitAuras(unitID.."target", "HARMFUL")
		end
	elseif event == "UNIT_AURA" then
		local unitID = ...
		if UnitExists(unitID) then
			CheckUnitAuras(unitID, "HELPFUL")
			CheckUnitAuras(unitID, "HARMFUL")
		end
	elseif event == "PLAYER_LOGOUT" then
		_G.LAI_DB = LAI_DB
	elseif event == "ADDON_LOADED" or event == "PLAYER_LOGIN" then
		LAI_DB = _G.LAI_DB or {}
		self:RegisterEvent('UPDATE_MOUSEOVER_UNIT')
		self:RegisterEvent('PLAYER_TARGET_CHANGED')
		self:RegisterEvent('PLAYER_LOGOUT')
		self:RegisterEvent('UNIT_TARGET')
		self:RegisterEvent('UNIT_AURA')
		
	end
end
lib.frame = lib.frame or CreateFrame("Frame")
lib.frame:SetScript("OnEvent", OnEvent)
lib.frame:RegisterEvent('ADDON_LOADED')
lib.frame:RegisterEvent('PLAYER_LOGIN')
--@end-debug@]===]

--Our spell info table.
lib.auraInfo = {
[20375] = "1800;0", --Seal of Command
[48470] = "3600;1", --Gift of the Wild (Rank 4)
[31583] = "0;0", --Arcane Empowerment (Rank 3)
[66846] = "0;0", --Ochre Skeletal Warhorse
[33660] = "0;0", --Swift Pink Hawkstrider
[55610] = "0;0", --Improved Icy Talons
[47893] = "1800;0", --Fel Armor (Rank 4)
[48266] = "0;0", --Blood Presence
[58777] = "0;0", --Mana Spring (Rank 8)
[64420] = "15;0", --Sniper Training (Rank 3)
[58655] = "0;0", --Flametongue Totem (Rank 8)
[16237] = "15;1", --Ancestral Fortitude (Rank 3)
[57821] = "0;0", --Champion of the Kirin Tor
[60116] = "0;0", --Armored Brown Bear
[53390] = "15;1", --Tidal Waves
[54043] = "0;0", --Retribution Aura (Rank 7)
[57822] = "0;0", --Wyrmrest Champion
[61847] = "0;0", --Aspect of the Dragonhawk (Rank 2)
[57332] = "3600;0", --Well Fed
[48932] = "1800;1", --Blessing of Might (Rank 10)
[2895] = "0;0", --Wrath of Air Totem
[22718] = "0;0", --Black War Kodo
[24932] = "0;0", --Leader of the Pack
[57399] = "3600;0", --Well Fed
[48162] = "3600;1", --Prayer of Fortitude (Rank 4)
[29801] = "0;0", --Rampage (Passive)
[49281] = "600;1", --Lightning Shield (Rank 11)
[48074] = "3600;1", --Prayer of Spirit (Rank 3)
[57623] = "180;0", --Horn of Winter (Rank 2)
[48263] = "0;0", --Frost Presence
[32223] = "0;0", --Crusader Aura
[48265] = "0;0", --Unholy Presence
[23251] = "0;0", --Swift Timber Wolf
[53138] = "0;0", --Abomination's Might (Rank 2)
[43002] = "3600;1", --Arcane Brilliance (Rank 3)
[25780] = "0;1", --Righteous Fury
[21084] = "1800;0", --Seal of Righteousness
[48170] = "1200;1", --Prayer of Shadow Protection (Rank 3)
[51470] = "0;0", --Elemental Oath (Rank 2)
[55531] = "0;0", --Mechano-hog
[768] = "0;0", --Cat Form (Shapeshift)
[25163] = "0;0", --Oozeling's Disgusting Aura
[53768] = "0;0", --Haunted
[22721] = "0;0", --Black War Raptor
[48420] = "0;0", --Master Shapeshifter
[15473] = "0;0", --Shadowform
[43046] = "1800;0", --Molten Armor (Rank 3)
[63944] = "60;0", --Renewed Hope
[588] = "1800;0", --Inner Fire (Rank 1)
[17481] = "0;0", --Rivendare's Deathcharger
[35025] = "0;0", --Swift Green Hawkstrider
[168] = "1800;0", --Frost Armor (Rank 1)
[75447] = "0;0", --Ferocious Inspiration
[1459] = "1800;1", --Arcane Intellect (Rank 1)
[1244] = "1800;1", --Power Word: Fortitude (Rank 2)
[57288] = "3600;0", --Well Fed
[57475] = "0;0", --Kindred Spirits
[22724] = "0;0", --Black War Wolf
[33721] = "3600;0", --Spellpower Elixir
[43900] = "0;0", --Swift Brewfest Ram
[57820] = "0;0", --Ebon Champion
[47436] = "360;0", --Battle Shout (Rank 9)
[2479] = "30;0", --Honorless Target
[72586] = "1800;1", --Blessing of Forgotten Kings
[57111] = "3600;0", --Well Fed
[60119] = "0;0", --Black War Bear
[57458] = "0;0", --Kindred Spirits
[61846] = "0;0", --Aspect of the Dragonhawk (Rank 1)
[67016] = "3600;0", --Flask of the North
[23161] = "0;0", --Dreadsteed (Summon)
[48778] = "0;0", --Acherus Deathcharger (Summon)
[72588] = "3600;1", --Gift of the Wild
[19746] = "0;0", --Concentration Aura
[30809] = "0;0", --Unleashed Rage (Rank 3)
[23509] = "0;0", --Frostwolf Howler
[59788] = "0;0", --Black War Mammoth
[1126] = "1800;1", --Mark of the Wild (Rank 1)
[19506] = "0;0", --Trueshot Aura
[43010] = "30;1", --Fire Ward (Rank 7)
[34767] = "0;0", --Summon Charger (Summon)
[72968] = "0;0", --Precious's Ribbon
[22722] = "0;0", --Red Skeletal Warhorse
[25899] = "1800;1", --Greater Blessing of Sanctuary
[27044] = "0;0", --Aspect of the Hawk (Rank 8)
[61465] = "0;0", --Grand Black War Mammoth
[59568] = "0;0", --Blue Drake
[4511] = "0;0", --Phase Shift
[75620] = "0;0", --Celestial Steed
[53758] = "3600;0", --Flask of Stoneblood
[20216] = "0;1", --Divine Favor
[57330] = "180;0", --Horn of Winter (Rank 1)
[42995] = "1800;1", --Arcane Intellect (Rank 7)
[57100] = "3600;0", --Well Fed
[28878] = "0;0", --Heroic Presence (Racial Passive)
[33891] = "0;0", --Tree of Life (Shapeshift)
[12544] = "1800;1", --Frost Armor
[58450] = "1800;1", --Agility (Rank 7)
[57819] = "0;0", --Argent Champion
[34074] = "0;0", --Aspect of the Viper
[1243] = "1800;1", --Power Word: Fortitude (Rank 1)
[57327] = "3600;0", --Well Fed
[9634] = "0;0", --Dire Bear Form (Shapeshift)
[24858] = "0;0", --Moonkin Form (Shapeshift)
[23338] = "0;0", --Swift Stormsaber
[20165] = "1800;0", --Seal of Light
[48469] = "1800;1", --Mark of the Wild (Rank 9)
[43020] = "60;1", --Mana Shield (Rank 9)
[35027] = "0;0", --Swift Purple Hawkstrider
[32999] = "3600;1", --Prayer of Spirit (Rank 2)
[25392] = "3600;1", --Prayer of Fortitude (Rank 3)
[130] = "30;1", --Slow Fall
[55629] = "0;0", --First Lieutenant
[23247] = "0;0", --Great White Kodo
[53736] = "1800;0", --Seal of Corruption
[48418] = "0;0", --Master Shapeshifter
[48422] = "0;0", --Master Shapeshifter
[48934] = "1800;1", --Greater Blessing of Might (Rank 5)
[48938] = "1800;1", --Greater Blessing of Wisdom (Rank 5)
[48942] = "0;0", --Devotion Aura (Rank 10)
[5118] = "0;0", --Aspect of the Cheetah
[48073] = "1800;1", --Divine Spirit (Rank 6)
[53752] = "7200;0", --Lesser Flask of Toughness
[17619] = "0;0", --Alchemist's Stone
[71184] = "15;0", --Soothing
[34123] = "0;0", --Tree of Life (Passive)
[642] = "12;1", --Divine Shield
[44151] = "0;0", --Turbo-Charged Flying Machine
[21850] = "3600;1", --Gift of the Wild (Rank 2)
[49379] = "0;0", --Great Brewfest Kodo
[59797] = "0;0", --Ice Mammoth
[57960] = "600;1", --Water Shield (Rank 9)
[71220] = "15;0", --Energized
[25898] = "1800;1", --Greater Blessing of Kings
[23243] = "0;0", --Swift Orange Raptor
[43008] = "1800;0", --Ice Armor (Rank 6)
[67696] = "10;0", --Energized
[26992] = "3600;1", --Thorns (Rank 7)
[48161] = "1800;1", --Power Word: Fortitude (Rank 8)
[24907] = "0;0", --Moonkin Aura
[47982] = "0;0", --Blood Pact (Rank 7)
[7353] = "60;0", --Cozy Fire
[57940] = "0;0", --Essence of Wintergrasp
[6562] = "0;0", --Heroic Presence (Racial Passive)
[54646] = "1800;1", --Focus Magic
[44153] = "0;0", --Flying Machine
[9884] = "1800;1", --Mark of the Wild (Rank 6)
[57325] = "3600;0", --Well Fed
[48441] = "18;1", --Rejuvenation (Rank 15)
[15286] = "1800;0", --Vampiric Embrace
[55166] = "20;1", --Tidal Force
[70806] = "10;0", --Rapid Currents
[61301] = "15;1", --Riptide (Rank 4)
[48421] = "0;0", --Master Shapeshifter
[58744] = "0;0", --Frost Resistance (Rank 6)
[58754] = "0;0", --Stoneskin (Rank 10)
[48168] = "1800;0", --Inner Fire (Rank 9)
[24529] = "0;0", --Spirit Bond (Rank 2)
[65639] = "0;0", --Swift Red Hawkstrider
[63635] = "0;0", --Darkspear Raptor
[49222] = "300;0", --Bone Shield
[43024] = "1800;0", --Mage Armor (Rank 6)
[61316] = "3600;1", --Dalaran Brilliance (Rank 3)
[25389] = "1800;1", --Power Word: Fortitude (Rank 7)
[25312] = "1800;1", --Divine Spirit (Rank 5)
[22720] = "0;0", --Black War Ram
[23250] = "0;0", --Swift Brown Wolf
[68054] = "600;0", --Pressing Engagement
[57294] = "3600;0", --Well Fed
[54753] = "0;0", --White Polar Bear
[57358] = "3600;0", --Well Fed
[60025] = "0;0", --Albino Drake
[20217] = "600;1", --Blessing of Kings
[2645] = "0;1", --Ghost Wolf
[7301] = "3600;0", --Frost Armor (Rank 3)
[60118] = "0;0", --Black War Bear
[48945] = "0;0", --Frost Resistance Aura (Rank 5)
[18499] = "10;0", --Berserker Rage
[57567] = "0;0", --Fel Intelligence (Rank 5)
[53806] = "600;0", --Pygmy Oil
[57531] = "0;1", --Arcane Potency
[23246] = "0;0", --Purple Skeletal Warhorse
[40625] = "5400;0", --Apexis Emanations
[53760] = "3600;0", --Flask of Endless Rage
[65641] = "0;0", --Great Golden Kodo
[48931] = "1800;1", --Blessing of Might (Rank 9)
[546] = "600;0", --Water Walking
[60002] = "0;0", --Time-Lost Proto-Drake
[72590] = "3600;1", --Fortitude
[46628] = "0;0", --Swift White Hawkstrider
[24252] = "0;0", --Swift Zulian Tiger
[48169] = "600;1", --Shadow Protection (Rank 5)
[53307] = "600;1", --Thorns (Rank 8)
[26991] = "3600;1", --Gift of the Wild (Rank 3)
[20166] = "1800;0", --Seal of Wisdom
[61573] = "0;0", --Banner of the Alliance
[10156] = "1800;1", --Arcane Intellect (Rank 4)
[57669] = "15;0", --Replenishment
[22782] = "1800;0", --Mage Armor (Rank 2)
[59578] = "15;1", --The Art of War (Rank 2)
[48018] = "360;0", --Demonic Circle: Summon
[60551] = "10;0", --Furious Gladiator's Libram of Fortitude
[20053] = "30;1", --Vengeance
[50449] = "30;1", --Bloody Vengeance
[8907] = "1800;1", --Mark of the Wild (Rank 5)
[47889] = "1800;0", --Demon Armor (Rank 8)
[10301] = "0;0", --Retribution Aura (Rank 5)
[19838] = "1800;1", --Blessing of Might (Rank 6)
[34769] = "0;0", --Summon Warhorse (Summon)
[20005] = "5;0", --Chilled
[49028] = "12;0", --Dancing Rune Weapon
[55078] = "15;2", --Blood Plague
[45524] = "10;1", --Chains of Ice
[55233] = "10;0", --Vampiric Blood
[55095] = "15;2", --Frost Fever
[1044] = "6;1", --Hand of Freedom
[20066] = "6;1", --Repentance
[25228] = "0;0", --Soul Link
[43265] = "0;0", --Death and Decay (Rank 1)
[19836] = "1800;1", --Blessing of Might (Rank 4)
[61840] = "8;1", --Righteous Vengeance
[48947] = "0;0", --Fire Resistance Aura (Rank 5)
[54499] = "20;0", --Heart of the Crusader (Rank 3)
[10308] = "6;1", --Hammer of Justice (Rank 4)
[49016] = "30;0", --Hysteria
[20184] = "10;1", --Judgement of Justice
[14308] = "10;1", --Freezing Trap Effect (Rank 2)
[20549] = "2;0", --War Stomp (Racial)
[60345] = "3600;0", --Armor Piercing
[10299] = "0;0", --Retribution Aura (Rank 3)
[47476] = "5;1", --Strangulate
[22888] = "7200;0", --Rallying Cry of the Dragonslayer
}